/* Program: ex2-9.c
 * ---------------------------------------------------------------------
 * Solution to exercise 9, series 2.
 *
 * This program encodes and decodes a text using Vigenere's
 * encryption.  The key is given as a command-line argument.
 * Decoding is possible by giving the flag "-d" before the key.
 * ---------------------------------------------------------------------
 * Nikolaos S. Papaspyrou (nickie@softlab.ntua.gr)
 * National Technical University of Athens
 * Department of Electrical Engineering
 * ---------------------------------------------------------------------
 * Course: Programming Techniques (2nd semester)
 * URL:    http://www.softlab.ntua.gr/~nickie/Courses/progtech/
 * ---------------------------------------------------------------------
 */


/* Header files */

#include <stdio.h>
#include <ctype.h>


/* Function to encode using p as key */

void encode (char p[])
{
    int n = strlen(p);
    int i = 0;
    int c;

    while ((c = getchar()) != EOF) {
        if (isalpha(c)) {
            int codec   = tolower(c) - 'a' + 1;
            int codep   = tolower(p[i]) - 'a' + 1;
            int codenew = codec + codep;

            if (codenew > 26)
                codenew -= 26;

            if (islower(c))
                c = codenew - 1 + 'a';
            else
                c = codenew - 1 + 'A';

            if (++i >= n)
                i = 0;
        }
        putchar(c);
    }
}


/* Function to decode using p as key: the reverse of the previous */

void decode (char p[])
{
    int n = strlen(p);
    int i = 0;
    int c;

    while ((c = getchar()) != EOF) {
        if (isalpha(c)) {
            int codec   = tolower(c) - 'a' + 1;
            int codep   = tolower(p[i]) - 'a' + 1;
            int codenew = codec - codep;

            if (codenew < 0)
                codenew += 26;

            if (islower(c))
                c = codenew - 1 + 'a';
            else
                c = codenew - 1 + 'A';

            if (++i >= n)
                i = 0;
        }
        putchar(c);
    }
}


/* Main program */

int main (int argc, char * argv[])
{
    /* Check if a key has been given */

    if (argc < 2) {
        printf("You forgot the key...\n");
        return 1;
    }

    /* Choose between encoding and decoding */

    if (argc > 2 && strcmp(argv[1], "-d") == 0)
        decode(argv[2]);
    else
        encode(argv[1]);

    return 0;
}
